/*  This file is part of hpafs (Hidden Protected Area FileSystem). hpafs is a
 *  FUSE (Filesystem in USErspace).
 *
 *  Copyright (C) 2006, 2007 Paul Bolle <pebolle@tiscali.nl>
 *
 *  hpafs is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  hpafs is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */

#include <stdio.h>
#include <unistd.h>

#include "beer.h"

int
read_beer(beer_t * beer, int fd, int debug)
{
    READ_BEER_MEMBER(signature)
    READ_BEER_MEMBER(size)
    READ_BEER_MEMBER(flags)
    READ_BEER_MEMBER(reported_cylinders)
    READ_BEER_MEMBER(reported_heads)
    READ_BEER_MEMBER(reported_sectors)
    READ_BEER_MEMBER(reported_bytes_sector)
    READ_BEER_MEMBER(reported_sectors_device)
    READ_BEER_MEMBER(formatted_cylinders)
    READ_BEER_MEMBER(formatted_heads)
    READ_BEER_MEMBER(formatted_sectors)
    READ_BEER_MEMBER(formatted_bytes_sector)
    READ_BEER_MEMBER(formatted_sectors_device)
    READ_BEER_MEMBER(year)
    READ_BEER_MEMBER(day)
    READ_BEER_MEMBER(timestamp)
    READ_BEER_MEMBER(zero0)
    READ_BEER_MEMBER(device_index)
    READ_BEER_MEMBER(hpa_start)
    READ_BEER_MEMBER(boot_code_address)
    READ_BEER_MEMBER(num_entries)
    READ_BEER_MEMBER(dir_length)
    READ_BEER_MEMBER(zero1)
    READ_BEER_MEMBER(revision)
    READ_BEER_MEMBER(device_name)
    READ_BEER_MEMBER(checksum)
    return (1);
}

int
parse_beer(beer_t * beer)
{
    int i;
    char year[5], day[4], dev_name[41];;

    fprintf(stdout, "beer.signature: 0x%hX\n", beer->signature);
    fprintf(stdout, "beer.size: %hd\n", beer->size);
    fprintf(stdout, "beer.flags: %hd\n", beer->flags);
    fprintf(stdout, "    Read Only:                           %s\n",
            beer->flags & BEER_RO ? "yes" : "");
    fprintf(stdout, "    Generated Record:                    %s\n", 
            beer->flags & BEER_GENERATED_RECORD ? "yes" : "");
    fprintf(stdout, "    Use Reserved Area Boot Code Address: %s\n",
            beer->flags & BEER_USE_RABCA ? "yes" : "");
    fprintf(stdout, "    Configuration Time Stamp is valid:   %s\n",
            beer->flags & BEER_TIME_STAMP_VALID ? "yes" : "");
    fprintf(stdout, "    Device Supports LBA:                 %s\n",
            beer->flags & BEER_DEVICE_SUPPORTS_LBA ? "yes" : "");
    fprintf(stdout, "    Directory of Services is Present     %s\n",
            beer->flags & BEER_DOS_PRESENT ? "yes" : "");
    fprintf(stdout, "    Formatted Geometry Valid             %s\n",
            beer->flags & BEER_FORMATTED_GEOM_VALID ? "yes" : "");
    fprintf(stdout, "    Reported Geometry Valid              %s\n",
            beer->flags & BEER_REPORTED_GEOM_VALID ? "yes" : "");
    fprintf(stdout, "beer.reported_cylinders: %d\n", beer->reported_cylinders);
    fprintf(stdout, "beer.reported_heads: %d\n", beer->reported_heads);
    fprintf(stdout, "beer.reported_sectors: %d\n", beer->reported_sectors);
    fprintf(stdout, "beer.reported_bytes_sector: %d\n", beer->reported_bytes_sector);
    fprintf(stdout, "beer.reported_sectors_device: %lld\n", beer->reported_sectors_device);
    fprintf(stdout, "beer.formatted_cylinders: %d\n", beer->formatted_cylinders);
    fprintf(stdout, "beer.formatted_heads: %d\n", beer->formatted_heads);
    fprintf(stdout, "beer.formatted_sectors: %d\n", beer->formatted_sectors);
    fprintf(stdout, "beer.formatted_bytes_sector: %d\n", beer->formatted_bytes_sector);
    fprintf(stdout, "beer.formatted_sectors_device: %lld\n", beer->formatted_sectors_device);
    for (i = 0; i < 4; i++)
        year[i] = (beer->year >> (12 - i * 4) & 15) + 48;
    year[4]= '\0';
    fprintf(stdout, "year: %s\n", year);
    for (i = 0; i < 3; i++)
        day[i] = (beer->day >> (8 - i * 4) & 15) + 48;
    day[3]= '\0';
    fprintf(stdout, "day: %s\n", day);
    fprintf(stdout, "beer.timestamp: %d\n", beer->timestamp);
    fprintf(stdout, "beer.device_index: 0x%hhX\n", beer->device_index);
    fprintf(stdout, "beer.hpa_start: %lld\n", beer->hpa_start);
    fprintf(stdout, "beer.boot_code_address: %lld\n", beer->boot_code_address);
    fprintf(stdout, "beer.num_entries: %hu\n", beer->num_entries);
    fprintf(stdout, "beer.dir_length: %hu\n", beer->dir_length);
    fprintf(stdout, "beer.revision: %hhu\n", beer->revision);
    for (i = 0; i < 40; i++)
        dev_name[i] = beer->device_name[i];
    dev_name[40]= '\0';
    fprintf(stdout, "dev_name: %s\n", dev_name);
    fprintf(stdout, "beer.checksum: %d\n", beer->checksum);
    return (1);
}

int
check_beer(int fd, int debug) 
{
    short tmp, sum = 0;
    int i;

    for (i = 0; i < (BEER_SIZE/2); i++){
        if (read(fd, &tmp, sizeof(short)) != (ssize_t) 2) {
            if (debug) fprintf(stderr, "check_beer read short read\n");
            return (0);
        }
        if (debug) fprintf(stderr, "tmp [%d]: %hd\n", i, tmp);
        sum += tmp;
        if (debug) fprintf(stderr, "sum [%d]: %hd\n", i, sum);
    }
    return (sum == 0);
}
