/*
Offset          Type            Description
0-1             Word            Signature = BEEFh
2-3             Word            BEER size. Shall be 128.
4-5             Word            Capabilities Word
                                Bit     Description
                                8-15    Reserved
                                7       Read Only
                                6       Generated Record
                                5       Use Reserved Area Boot Code Address
                                4       Configuration Time Stamp is valid
                                3       Device Supports LBA
                                2       Directory of Services is Present
                                1       Formatted Geometry Valid
                                0       Reported Geometry Valid
6-9             DWord           Reported Cylinders
10-13           DWord           Reported Heads
14-17           Dword           Reported Sectors
18-21           Dword           Reported Bytes/Sector
22-29           Qword           Reported Sectors/Device
30-33           Dword           Formatted Cylinders
34-37           Dword           Formatted Heads
38-41           Dword           Formatted Sectors
42-45           Dword           Formatted Bytes/Sector
46-53           Qword           Formatted Sectors/Device
54-55           Word            BCD Year
56-57           Word            Linear Day
58-61           Dword           Configuration Time stamp
62              Byte            Reserved        shall be 0
63              Byte            Device Index
64-71           Qword           Host Protected Area Start
72-79           Qword           Reserved Area Boot Code Address
80-81           Word            Number of entries in the BEER Directory of
                                Services
82-83           Word            Length of a BEER Directory of Services
                                Entry
84              Byte            Reserved shall be 0
85              Byte            Revision of the standard used to generate
                                this record
86-125          String          Device Name
126-127         Word            16 Bit Checksum
*/

#define BEER_SIGNATURE              0xBEEF
#define BEER_SIZE                   0x80

#define BEER_RO                     1 << 7
#define BEER_GENERATED_RECORD       1 << 6
#define BEER_USE_RABCA              1 << 5
#define BEER_TIME_STAMP_VALID       1 << 4
#define BEER_DEVICE_SUPPORTS_LBA    1 << 3
#define BEER_DOS_PRESENT            1 << 2
#define BEER_FORMATTED_GEOM_VALID   1 << 1
#define BEER_REPORTED_GEOM_VALID    1

#define READ_BEER_MEMBER(m)      if (read(fd, &beer->m, sizeof(beer->m)) != sizeof(beer->m)) {\
        if (debug) fprintf(stderr, "__FUNCTION__: short read\n");\
        return (0);\
    }

typedef struct beer {
        short           signature;      /* 0xBEEF */
        short           size;           /* 0x0080 */
        short           flags;
        int             reported_cylinders;
        int             reported_heads;
        int             reported_sectors;
        int             reported_bytes_sector;
        long long       reported_sectors_device;
        int             formatted_cylinders;
        int             formatted_heads;
        int             formatted_sectors;
        int             formatted_bytes_sector;
        long long       formatted_sectors_device;
        short           year;
        short           day;
        int             timestamp;
        char            zero0;
        char            device_index;
        long long       hpa_start;
        long long       boot_code_address;
        short           num_entries;
        short           dir_length;     /* 0x0040 */
        char            zero1;
        char            revision;
        char            device_name[40];
        short           checksum;
} beer_t;

int read_beer(beer_t * beer, int fd, int debug);
int parse_beer(beer_t * beer);
int check_beer(int fd, int debug);
