/*
Offset          Type            Description
0               Byte            Directory Flags
                                Bit     Description
                                7       Service area is available as B:
                                6       Reserved
                                5       Diagnostic Service
                                4       Service Area is Read Only
                                3       This Boot
                                2       Empty Service Area
                                1       Hidden Service Area
                                0       Service Area is bootable as A:
1               Byte            Reserved.
2-9             QWord           Service Area Start
10-17           QWord           Service Area Size
18-21           DWord           Load Sectors
22-25           DWord           Load Address
26-27           Word            Service Area ID
28-59           Byte            ID String
60-61           Word            Reserved
62-63           Word            16 bit Checksum
 */

#define DOS_LENGTH              0x40

#define DOS_DRIVE_B             1 << 7
#define DOS_DIAGNOSTIC          1 << 5
#define DOS_RO                  1 << 4
#define DOS_THIS_BOOT           1 << 3
#define DOS_EMPTY               1 << 2
#define DOS_HIDDEN              1 << 1
#define DOS_BOOTABLE_DRIVE_A    1

#define READ_DIRENTRY_MEMBER(m)      if (read(fd, &dos->m, sizeof(dos->m)) != sizeof(dos->m)) {\
        if (debug) fprintf(stderr, "__FUNCTION__: short read\n");\
        return (0);\
    }

typedef struct direntry {
        char            flags;
        char            zero0;
        long long       service_area_start;
        long long       service_area_size;
        int             load_sectors;
        int             load_address;
        short           service_area_id;
        char            id_string[32];
        short           zero1;
        short           checksum;
} direntry_t;

int read_dos(direntry_t * dos, int fd, int debug);
int parse_dos(direntry_t * dos, int num);
int check_dos(int fd, int debug);
